# The contents of this file are automatically written by
# tools/generate_schema_wrapper.py. Do not modify directly.
from . import core
from altair.utils import use_signature
from altair.utils.schemapi import Undefined


class MarkMethodMixin(object):
    """A mixin class that defines mark methods"""

    def mark_arc(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                 ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                 baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                 color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                 cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                 cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                 cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined, dir=Undefined,
                 discreteBandSize=Undefined, dx=Undefined, dy=Undefined, ellipsis=Undefined,
                 fill=Undefined, fillOpacity=Undefined, filled=Undefined, font=Undefined,
                 fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined, height=Undefined,
                 href=Undefined, innerRadius=Undefined, interpolate=Undefined, invalid=Undefined,
                 limit=Undefined, line=Undefined, lineBreak=Undefined, lineHeight=Undefined,
                 opacity=Undefined, order=Undefined, orient=Undefined, outerRadius=Undefined,
                 padAngle=Undefined, point=Undefined, radius=Undefined, radius2=Undefined,
                 radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined, size=Undefined,
                 smooth=Undefined, stroke=Undefined, strokeCap=Undefined, strokeDash=Undefined,
                 strokeDashOffset=Undefined, strokeJoin=Undefined, strokeMiterLimit=Undefined,
                 strokeOffset=Undefined, strokeOpacity=Undefined, strokeWidth=Undefined,
                 style=Undefined, tension=Undefined, text=Undefined, theta=Undefined, theta2=Undefined,
                 theta2Offset=Undefined, thetaOffset=Undefined, thickness=Undefined,
                 timeUnitBand=Undefined, timeUnitBandPosition=Undefined, tooltip=Undefined,
                 url=Undefined, width=Undefined, x=Undefined, x2=Undefined, x2Offset=Undefined,
                 xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined, yOffset=Undefined,
                 **kwds):
        """Set the chart's mark to 'arc'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="arc", **kwds)
        else:
            copy.mark = "arc"
        return copy

    def mark_area(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                  ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                  baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                  color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                  cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                  cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                  cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                  dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                  ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                  font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                  height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                  invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                  lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                  outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                  radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                  size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                  strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                  strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                  strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                  theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                  thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                  tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                  x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                  yOffset=Undefined, **kwds):
        """Set the chart's mark to 'area'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="area", **kwds)
        else:
            copy.mark = "area"
        return copy

    def mark_bar(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                 ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                 baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                 color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                 cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                 cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                 cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined, dir=Undefined,
                 discreteBandSize=Undefined, dx=Undefined, dy=Undefined, ellipsis=Undefined,
                 fill=Undefined, fillOpacity=Undefined, filled=Undefined, font=Undefined,
                 fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined, height=Undefined,
                 href=Undefined, innerRadius=Undefined, interpolate=Undefined, invalid=Undefined,
                 limit=Undefined, line=Undefined, lineBreak=Undefined, lineHeight=Undefined,
                 opacity=Undefined, order=Undefined, orient=Undefined, outerRadius=Undefined,
                 padAngle=Undefined, point=Undefined, radius=Undefined, radius2=Undefined,
                 radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined, size=Undefined,
                 smooth=Undefined, stroke=Undefined, strokeCap=Undefined, strokeDash=Undefined,
                 strokeDashOffset=Undefined, strokeJoin=Undefined, strokeMiterLimit=Undefined,
                 strokeOffset=Undefined, strokeOpacity=Undefined, strokeWidth=Undefined,
                 style=Undefined, tension=Undefined, text=Undefined, theta=Undefined, theta2=Undefined,
                 theta2Offset=Undefined, thetaOffset=Undefined, thickness=Undefined,
                 timeUnitBand=Undefined, timeUnitBandPosition=Undefined, tooltip=Undefined,
                 url=Undefined, width=Undefined, x=Undefined, x2=Undefined, x2Offset=Undefined,
                 xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined, yOffset=Undefined,
                 **kwds):
        """Set the chart's mark to 'bar'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="bar", **kwds)
        else:
            copy.mark = "bar"
        return copy

    def mark_image(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                   ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                   baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                   color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                   cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                   cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                   cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                   dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                   ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                   font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                   height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                   invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                   lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                   outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                   radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                   size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                   strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                   strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                   strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                   theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                   thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                   tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                   x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                   yOffset=Undefined, **kwds):
        """Set the chart's mark to 'image'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="image", **kwds)
        else:
            copy.mark = "image"
        return copy

    def mark_line(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                  ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                  baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                  color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                  cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                  cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                  cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                  dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                  ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                  font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                  height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                  invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                  lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                  outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                  radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                  size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                  strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                  strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                  strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                  theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                  thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                  tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                  x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                  yOffset=Undefined, **kwds):
        """Set the chart's mark to 'line'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="line", **kwds)
        else:
            copy.mark = "line"
        return copy

    def mark_point(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                   ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                   baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                   color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                   cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                   cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                   cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                   dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                   ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                   font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                   height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                   invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                   lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                   outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                   radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                   size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                   strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                   strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                   strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                   theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                   thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                   tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                   x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                   yOffset=Undefined, **kwds):
        """Set the chart's mark to 'point'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="point", **kwds)
        else:
            copy.mark = "point"
        return copy

    def mark_rect(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                  ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                  baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                  color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                  cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                  cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                  cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                  dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                  ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                  font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                  height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                  invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                  lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                  outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                  radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                  size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                  strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                  strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                  strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                  theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                  thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                  tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                  x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                  yOffset=Undefined, **kwds):
        """Set the chart's mark to 'rect'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="rect", **kwds)
        else:
            copy.mark = "rect"
        return copy

    def mark_rule(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                  ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                  baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                  color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                  cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                  cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                  cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                  dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                  ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                  font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                  height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                  invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                  lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                  outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                  radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                  size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                  strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                  strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                  strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                  theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                  thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                  tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                  x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                  yOffset=Undefined, **kwds):
        """Set the chart's mark to 'rule'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="rule", **kwds)
        else:
            copy.mark = "rule"
        return copy

    def mark_text(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                  ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                  baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                  color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                  cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                  cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                  cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                  dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                  ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                  font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                  height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                  invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                  lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                  outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                  radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                  size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                  strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                  strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                  strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                  theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                  thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                  tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                  x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                  yOffset=Undefined, **kwds):
        """Set the chart's mark to 'text'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="text", **kwds)
        else:
            copy.mark = "text"
        return copy

    def mark_tick(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                  ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                  baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                  color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                  cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                  cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                  cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                  dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                  ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                  font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                  height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                  invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                  lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                  outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                  radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                  size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                  strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                  strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                  strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                  theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                  thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                  tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                  x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                  yOffset=Undefined, **kwds):
        """Set the chart's mark to 'tick'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="tick", **kwds)
        else:
            copy.mark = "tick"
        return copy

    def mark_trail(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                   ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                   baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                   color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                   cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                   cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                   cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                   dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                   ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                   font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                   height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                   invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                   lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                   outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                   radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                   size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                   strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                   strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                   strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                   theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                   thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                   tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                   x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                   yOffset=Undefined, **kwds):
        """Set the chart's mark to 'trail'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="trail", **kwds)
        else:
            copy.mark = "trail"
        return copy

    def mark_circle(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                    ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                    baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                    color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                    cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                    cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                    cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                    dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                    ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                    font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                    height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                    invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                    lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                    outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                    radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                    size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                    strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                    strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                    strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                    theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                    thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                    tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                    x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined,
                    y2Offset=Undefined, yOffset=Undefined, **kwds):
        """Set the chart's mark to 'circle'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="circle", **kwds)
        else:
            copy.mark = "circle"
        return copy

    def mark_square(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                    ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                    baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                    color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                    cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                    cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                    cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                    dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                    ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                    font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                    height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                    invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                    lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                    outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                    radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined, shape=Undefined,
                    size=Undefined, smooth=Undefined, stroke=Undefined, strokeCap=Undefined,
                    strokeDash=Undefined, strokeDashOffset=Undefined, strokeJoin=Undefined,
                    strokeMiterLimit=Undefined, strokeOffset=Undefined, strokeOpacity=Undefined,
                    strokeWidth=Undefined, style=Undefined, tension=Undefined, text=Undefined,
                    theta=Undefined, theta2=Undefined, theta2Offset=Undefined, thetaOffset=Undefined,
                    thickness=Undefined, timeUnitBand=Undefined, timeUnitBandPosition=Undefined,
                    tooltip=Undefined, url=Undefined, width=Undefined, x=Undefined, x2=Undefined,
                    x2Offset=Undefined, xOffset=Undefined, y=Undefined, y2=Undefined,
                    y2Offset=Undefined, yOffset=Undefined, **kwds):
        """Set the chart's mark to 'square'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="square", **kwds)
        else:
            copy.mark = "square"
        return copy

    def mark_geoshape(self, align=Undefined, angle=Undefined, aria=Undefined, ariaRole=Undefined,
                      ariaRoleDescription=Undefined, aspect=Undefined, bandSize=Undefined,
                      baseline=Undefined, binSpacing=Undefined, blend=Undefined, clip=Undefined,
                      color=Undefined, continuousBandSize=Undefined, cornerRadius=Undefined,
                      cornerRadiusBottomLeft=Undefined, cornerRadiusBottomRight=Undefined,
                      cornerRadiusEnd=Undefined, cornerRadiusTopLeft=Undefined,
                      cornerRadiusTopRight=Undefined, cursor=Undefined, description=Undefined,
                      dir=Undefined, discreteBandSize=Undefined, dx=Undefined, dy=Undefined,
                      ellipsis=Undefined, fill=Undefined, fillOpacity=Undefined, filled=Undefined,
                      font=Undefined, fontSize=Undefined, fontStyle=Undefined, fontWeight=Undefined,
                      height=Undefined, href=Undefined, innerRadius=Undefined, interpolate=Undefined,
                      invalid=Undefined, limit=Undefined, line=Undefined, lineBreak=Undefined,
                      lineHeight=Undefined, opacity=Undefined, order=Undefined, orient=Undefined,
                      outerRadius=Undefined, padAngle=Undefined, point=Undefined, radius=Undefined,
                      radius2=Undefined, radius2Offset=Undefined, radiusOffset=Undefined,
                      shape=Undefined, size=Undefined, smooth=Undefined, stroke=Undefined,
                      strokeCap=Undefined, strokeDash=Undefined, strokeDashOffset=Undefined,
                      strokeJoin=Undefined, strokeMiterLimit=Undefined, strokeOffset=Undefined,
                      strokeOpacity=Undefined, strokeWidth=Undefined, style=Undefined,
                      tension=Undefined, text=Undefined, theta=Undefined, theta2=Undefined,
                      theta2Offset=Undefined, thetaOffset=Undefined, thickness=Undefined,
                      timeUnitBand=Undefined, timeUnitBandPosition=Undefined, tooltip=Undefined,
                      url=Undefined, width=Undefined, x=Undefined, x2=Undefined, x2Offset=Undefined,
                      xOffset=Undefined, y=Undefined, y2=Undefined, y2Offset=Undefined,
                      yOffset=Undefined, **kwds):
        """Set the chart's mark to 'geoshape'
    
        For information on additional arguments, see :class:`MarkDef`
        """
        kwds = dict(align=align, angle=angle, aria=aria, ariaRole=ariaRole,
                    ariaRoleDescription=ariaRoleDescription, aspect=aspect, bandSize=bandSize,
                    baseline=baseline, binSpacing=binSpacing, blend=blend, clip=clip, color=color,
                    continuousBandSize=continuousBandSize, cornerRadius=cornerRadius,
                    cornerRadiusBottomLeft=cornerRadiusBottomLeft,
                    cornerRadiusBottomRight=cornerRadiusBottomRight, cornerRadiusEnd=cornerRadiusEnd,
                    cornerRadiusTopLeft=cornerRadiusTopLeft, cornerRadiusTopRight=cornerRadiusTopRight,
                    cursor=cursor, description=description, dir=dir, discreteBandSize=discreteBandSize,
                    dx=dx, dy=dy, ellipsis=ellipsis, fill=fill, fillOpacity=fillOpacity, filled=filled,
                    font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight,
                    height=height, href=href, innerRadius=innerRadius, interpolate=interpolate,
                    invalid=invalid, limit=limit, line=line, lineBreak=lineBreak, lineHeight=lineHeight,
                    opacity=opacity, order=order, orient=orient, outerRadius=outerRadius,
                    padAngle=padAngle, point=point, radius=radius, radius2=radius2,
                    radius2Offset=radius2Offset, radiusOffset=radiusOffset, shape=shape, size=size,
                    smooth=smooth, stroke=stroke, strokeCap=strokeCap, strokeDash=strokeDash,
                    strokeDashOffset=strokeDashOffset, strokeJoin=strokeJoin,
                    strokeMiterLimit=strokeMiterLimit, strokeOffset=strokeOffset,
                    strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, style=style, tension=tension,
                    text=text, theta=theta, theta2=theta2, theta2Offset=theta2Offset,
                    thetaOffset=thetaOffset, thickness=thickness, timeUnitBand=timeUnitBand,
                    timeUnitBandPosition=timeUnitBandPosition, tooltip=tooltip, url=url, width=width,
                    x=x, x2=x2, x2Offset=x2Offset, xOffset=xOffset, y=y, y2=y2, y2Offset=y2Offset,
                    yOffset=yOffset, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.MarkDef(type="geoshape", **kwds)
        else:
            copy.mark = "geoshape"
        return copy

    def mark_boxplot(self, box=Undefined, clip=Undefined, color=Undefined, extent=Undefined,
                     median=Undefined, opacity=Undefined, orient=Undefined, outliers=Undefined,
                     rule=Undefined, size=Undefined, ticks=Undefined, **kwds):
        """Set the chart's mark to 'boxplot'
    
        For information on additional arguments, see :class:`BoxPlotDef`
        """
        kwds = dict(box=box, clip=clip, color=color, extent=extent, median=median, opacity=opacity,
                    orient=orient, outliers=outliers, rule=rule, size=size, ticks=ticks, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.BoxPlotDef(type="boxplot", **kwds)
        else:
            copy.mark = "boxplot"
        return copy

    def mark_errorbar(self, clip=Undefined, color=Undefined, extent=Undefined, opacity=Undefined,
                      orient=Undefined, rule=Undefined, size=Undefined, thickness=Undefined,
                      ticks=Undefined, **kwds):
        """Set the chart's mark to 'errorbar'
    
        For information on additional arguments, see :class:`ErrorBarDef`
        """
        kwds = dict(clip=clip, color=color, extent=extent, opacity=opacity, orient=orient, rule=rule,
                    size=size, thickness=thickness, ticks=ticks, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.ErrorBarDef(type="errorbar", **kwds)
        else:
            copy.mark = "errorbar"
        return copy

    def mark_errorband(self, band=Undefined, borders=Undefined, clip=Undefined, color=Undefined,
                       extent=Undefined, interpolate=Undefined, opacity=Undefined, orient=Undefined,
                       tension=Undefined, **kwds):
        """Set the chart's mark to 'errorband'
    
        For information on additional arguments, see :class:`ErrorBandDef`
        """
        kwds = dict(band=band, borders=borders, clip=clip, color=color, extent=extent,
                    interpolate=interpolate, opacity=opacity, orient=orient, tension=tension, **kwds)
        copy = self.copy(deep=False)
        if any(val is not Undefined for val in kwds.values()):
            copy.mark = core.ErrorBandDef(type="errorband", **kwds)
        else:
            copy.mark = "errorband"
        return copy


class ConfigMethodMixin(object):
    """A mixin class that defines config methods"""

    @use_signature(core.Config)
    def configure(self, *args, **kwargs):
        copy = self.copy(deep=False)
        copy.config = core.Config(*args, **kwargs)
        return copy

    @use_signature(core.RectConfig)
    def configure_arc(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["arc"] = core.RectConfig(*args, **kwargs)
        return copy

    @use_signature(core.AreaConfig)
    def configure_area(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["area"] = core.AreaConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axis(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axis"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisBand(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisBand"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisBottom(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisBottom"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisDiscrete(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisDiscrete"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisLeft(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisLeft"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisPoint(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisPoint"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisQuantitative(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisQuantitative"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisRight(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisRight"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisTemporal(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisTemporal"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisTop(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisTop"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisX(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisX"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisXBand(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisXBand"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisXDiscrete(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisXDiscrete"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisXPoint(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisXPoint"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisXQuantitative(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisXQuantitative"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisXTemporal(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisXTemporal"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisY(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisY"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisYBand(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisYBand"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisYDiscrete(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisYDiscrete"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisYPoint(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisYPoint"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisYQuantitative(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisYQuantitative"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.AxisConfig)
    def configure_axisYTemporal(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["axisYTemporal"] = core.AxisConfig(*args, **kwargs)
        return copy

    @use_signature(core.BarConfig)
    def configure_bar(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["bar"] = core.BarConfig(*args, **kwargs)
        return copy

    @use_signature(core.BoxPlotConfig)
    def configure_boxplot(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["boxplot"] = core.BoxPlotConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_circle(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["circle"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.CompositionConfig)
    def configure_concat(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["concat"] = core.CompositionConfig(*args, **kwargs)
        return copy

    @use_signature(core.ErrorBandConfig)
    def configure_errorband(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["errorband"] = core.ErrorBandConfig(*args, **kwargs)
        return copy

    @use_signature(core.ErrorBarConfig)
    def configure_errorbar(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["errorbar"] = core.ErrorBarConfig(*args, **kwargs)
        return copy

    @use_signature(core.CompositionConfig)
    def configure_facet(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["facet"] = core.CompositionConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_geoshape(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["geoshape"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.HeaderConfig)
    def configure_header(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["header"] = core.HeaderConfig(*args, **kwargs)
        return copy

    @use_signature(core.HeaderConfig)
    def configure_headerColumn(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["headerColumn"] = core.HeaderConfig(*args, **kwargs)
        return copy

    @use_signature(core.HeaderConfig)
    def configure_headerFacet(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["headerFacet"] = core.HeaderConfig(*args, **kwargs)
        return copy

    @use_signature(core.HeaderConfig)
    def configure_headerRow(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["headerRow"] = core.HeaderConfig(*args, **kwargs)
        return copy

    @use_signature(core.RectConfig)
    def configure_image(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["image"] = core.RectConfig(*args, **kwargs)
        return copy

    @use_signature(core.LegendConfig)
    def configure_legend(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["legend"] = core.LegendConfig(*args, **kwargs)
        return copy

    @use_signature(core.LineConfig)
    def configure_line(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["line"] = core.LineConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_mark(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["mark"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_point(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["point"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.ProjectionConfig)
    def configure_projection(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["projection"] = core.ProjectionConfig(*args, **kwargs)
        return copy

    @use_signature(core.RangeConfig)
    def configure_range(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["range"] = core.RangeConfig(*args, **kwargs)
        return copy

    @use_signature(core.RectConfig)
    def configure_rect(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["rect"] = core.RectConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_rule(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["rule"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.ScaleConfig)
    def configure_scale(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["scale"] = core.ScaleConfig(*args, **kwargs)
        return copy

    @use_signature(core.SelectionConfig)
    def configure_selection(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["selection"] = core.SelectionConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_square(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["square"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.MarkConfig)
    def configure_text(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["text"] = core.MarkConfig(*args, **kwargs)
        return copy

    @use_signature(core.TickConfig)
    def configure_tick(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["tick"] = core.TickConfig(*args, **kwargs)
        return copy

    @use_signature(core.TitleConfig)
    def configure_title(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["title"] = core.TitleConfig(*args, **kwargs)
        return copy

    @use_signature(core.LineConfig)
    def configure_trail(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["trail"] = core.LineConfig(*args, **kwargs)
        return copy

    @use_signature(core.ViewConfig)
    def configure_view(self, *args, **kwargs):
        copy = self.copy(deep=['config'])
        if copy.config is Undefined:
            copy.config = core.Config()
        copy.config["view"] = core.ViewConfig(*args, **kwargs)
        return copy


"""
Interactive Scatter Plot and Linked Layered Histogram
=====================================================

This example shows how to link a scatter plot and a histogram
together such that clicking on a point in the scatter plot will
isolate the distribution corresponding to that point, and vice versa.
"""
# category: interactive charts

import altair as alt
import pandas as pd
import numpy as np

# generate fake data
source = pd.DataFrame({'gender': ['M']*1000 + ['F']*1000,
               'height':np.concatenate((np.random.normal(69, 7, 1000),
                                       np.random.normal(64, 6, 1000))),
               'weight': np.concatenate((np.random.normal(195.8, 144, 1000),
                                        np.random.normal(167, 100, 1000))),
               'age': np.concatenate((np.random.normal(45, 8, 1000),
                                        np.random.normal(51, 6, 1000)))
        })

selector = alt.selection_single(empty='all', fields=['gender'])

color_scale = alt.Scale(domain=['M', 'F'],
                        range=['#1FC3AA', '#8624F5'])

base = alt.Chart(source).properties(
    width=250,
    height=250
).add_selection(selector)

points = base.mark_point(filled=True, size=200).encode(
    x=alt.X('mean(height):Q',
            scale=alt.Scale(domain=[0,84])),
    y=alt.Y('mean(weight):Q',
            scale=alt.Scale(domain=[0,250])),
    color=alt.condition(selector,
                        'gender:N',
                        alt.value('lightgray'),
                        scale=color_scale),
)

hists = base.mark_bar(opacity=0.5, thickness=100).encode(
    x=alt.X('age',
            bin=alt.Bin(step=5), # step keeps bin size the same
            scale=alt.Scale(domain=[0,100])),
    y=alt.Y('count()',
            stack=None,
            scale=alt.Scale(domain=[0,350])),
    color=alt.Color('gender:N',
                    scale=color_scale)
).transform_filter(
    selector
)


points | hists



# flake8: noqa
import warnings
from ...utils.deprecation import AltairDeprecationWarning

from .schema import *
from .api import *
from ._deprecated import *

from ...datasets import list_datasets, load_dataset

from ... import expr
from ...expr import datum

from .display import VegaLite, renderers

from .data import (
    MaxRowsError,
    pipe,
    curry,
    limit_rows,
    sample,
    to_json,
    to_csv,
    to_values,
    default_data_transformer,
    data_transformers,
)

warnings.warn(
    "The module altair.vegalite.v3 is deprecated and will be removed in Altair 5. "
    "Use `import altair as alt` instead of `import altair.vegalite.v3 as alt`.",
    AltairDeprecationWarning,
)



"""
Scatter Plot with Minimap
-------------------------
This example shows how to create a miniature version of a plot
such that creating a selection in the miniature version
adjusts the axis limits in another, more detailed view.
"""
# category: scatter plots

import altair as alt
from vega_datasets import data

source = data.seattle_weather()

zoom = alt.selection_interval(encodings=["x", "y"])

minimap = (
    alt.Chart(source)
    .mark_point()
    .add_selection(zoom)
    .encode(
        x="date:T",
        y="temp_max:Q",
        color=alt.condition(zoom, "weather", alt.value("lightgray")),
    )
    .properties(
        width=200,
        height=200,
        title="Minimap -- click and drag to zoom in the detail view",
    )
)

detail = (
    alt.Chart(source)
    .mark_point()
    .encode(
        x=alt.X(
            "date:T", scale=alt.Scale(domain={"selection": zoom.name, "encoding": "x"})
        ),
        y=alt.Y(
            "temp_max:Q",
            scale=alt.Scale(domain={"selection": zoom.name, "encoding": "y"}),
        ),
        color="weather",
    )
    .properties(width=600, height=400, title="Seattle weather -- detail view")
)

detail | minimap



"""
Line Chart with Points
----------------------
This chart shows a simple line chart with points marking each value. Use
``point=True`` for points with default appearance or customize it with
``OverlayMarkDef()``.
"""
# category: line charts
import altair as alt
import numpy as np
import pandas as pd

x = np.arange(100)
source = pd.DataFrame({
  'x': x,
  'f(x)': np.sin(x / 5)
})

alt.Chart(source).mark_line(
    point=alt.OverlayMarkDef(color="red")
).encode(
    x='x',
    y='f(x)'
)



from .html import spec_to_html


def spec_to_mimebundle(
    spec,
    format,
    mode=None,
    vega_version=None,
    vegaembed_version=None,
    vegalite_version=None,
    **kwargs,
):
    """Convert a vega/vega-lite specification to a mimebundle

    The mimebundle type is controlled by the ``format`` argument, which can be
    one of the following ['html', 'json', 'png', 'svg', 'pdf', 'vega', 'vega-lite']

    Parameters
    ----------
    spec : dict
        a dictionary representing a vega-lite plot spec
    format : string {'html', 'json', 'png', 'svg', 'pdf', 'vega', 'vega-lite'}
        the file format to be saved.
    mode : string {'vega', 'vega-lite'}
        The rendering mode.
    vega_version : string
        The version of vega.js to use
    vegaembed_version : string
        The version of vegaembed.js to use
    vegalite_version : string
        The version of vegalite.js to use. Only required if mode=='vega-lite'
    **kwargs :
        Additional arguments will be passed to the generating function

    Returns
    -------
    output : dict
        a mime-bundle representing the image

    Note
    ----
    The png, svg, pdf, and vega outputs require the altair_saver package
    to be installed.
    """
    if mode not in ["vega", "vega-lite"]:
        raise ValueError("mode must be either 'vega' or 'vega-lite'")

    if mode == "vega" and format == "vega":
        if vega_version is None:
            raise ValueError("Must specify vega_version")
        return {"application/vnd.vega.v{}+json".format(vega_version[0]): spec}
    if format in ["png", "svg", "pdf", "vega"]:
        try:
            import altair_saver
        except ImportError:
            raise ValueError(
                "Saving charts in {fmt!r} format requires the altair_saver package: "
                "see http://github.com/altair-viz/altair_saver/".format(fmt=format)
            )
        return altair_saver.render(spec, format, mode=mode, **kwargs)
    if format == "html":
        html = spec_to_html(
            spec,
            mode=mode,
            vega_version=vega_version,
            vegaembed_version=vegaembed_version,
            vegalite_version=vegalite_version,
            **kwargs,
        )
        return {"text/html": html}
    if format == "vega-lite":
        assert mode == "vega-lite"  # sanity check: should never be False
        if mode == "vega":
            raise ValueError("Cannot convert a vega spec to vegalite")
        if vegalite_version is None:
            raise ValueError("Must specify vegalite_version")
        return {"application/vnd.vegalite.v{}+json".format(vegalite_version[0]): spec}
    if format == "json":
        return {"application/json": spec}
    raise ValueError(
        "format must be one of "
        "['html', 'json', 'png', 'svg', 'pdf', 'vega', 'vega-lite']"
    )



"""
Wilkinson Dot Plot
------------------
An example of a `Wilkinson Dot Plot <https://en.wikipedia.org/wiki/Dot_plot_(statistics)>`_
"""
# category: other charts

import altair as alt
import pandas as pd

source = pd.DataFrame(
    {"data":[1,1,1,1,1,1,1,1,1,1,
             2,2,2,
             3,3,
             4,4,4,4,4,4]
    }
)

alt.Chart(source).mark_circle(opacity=1).transform_window(
    id='rank()', 
    groupby=['data']
).encode(
    alt.X('data:O'), 
    alt.Y('id:O', 
          axis=None, 
          sort='descending')
).properties(height=100)



import pandas as pd
from toolz import curried
from ..utils.core import sanitize_dataframe
from ..utils.data import (
    MaxRowsError,
    curry,
    pipe,
    sample,
    to_csv,
    to_json,
    to_values,
    check_data_type,
)


@curried.curry
def limit_rows(data, max_rows=5000):
    """Raise MaxRowsError if the data model has more than max_rows."""
    if not isinstance(data, (list, pd.DataFrame)):
        raise TypeError("Expected dict or DataFrame, got: {}".format(type(data)))
    if len(data) > max_rows:
        raise MaxRowsError(
            "The number of rows in your dataset is greater than the max of {}".format(
                max_rows
            )
        )
    return data


@curried.curry
def default_data_transformer(data):
    return curried.pipe(data, limit_rows, to_values)


__all__ = (
    "MaxRowsError",
    "curry",
    "default_data_transformer",
    "limit_rows",
    "pipe",
    "sanitize_dataframe",
    "sample",
    "to_csv",
    "to_json",
    "to_values",
    "check_data_type",
)



import os

from ...utils import PluginRegistry
from ...utils.deprecation import deprecated
from ..display import Displayable
from ..display import default_renderer_base
from ..display import json_renderer_base
from ..display import RendererType
from ..display import HTMLRenderer

from .schema import SCHEMA_VERSION

VEGA_VERSION = SCHEMA_VERSION.lstrip("v")
VEGAEMBED_VERSION = "3"


# ==============================================================================
# Vega 5 renderer logic
# ==============================================================================


# The MIME type for Vega 5 releases.
VEGA_MIME_TYPE = "application/vnd.vega.v5+json"  # type: str

# The entry point group that can be used by other packages to declare other
# renderers that will be auto-detected. Explicit registration is also
# allowed by the PluginRegistery API.
ENTRY_POINT_GROUP = "altair.vega.v5.renderer"  # type: str

# The display message when rendering fails
DEFAULT_DISPLAY = """\
<Vega 5 object>

If you see this message, it means the renderer has not been properly enabled
for the frontend that you are using. For more information, see
https://altair-viz.github.io/user_guide/troubleshooting.html
"""

renderers = PluginRegistry[RendererType](entry_point_group=ENTRY_POINT_GROUP)


here = os.path.dirname(os.path.realpath(__file__))


def default_renderer(spec):
    return default_renderer_base(spec, VEGA_MIME_TYPE, DEFAULT_DISPLAY)


def json_renderer(spec):
    return json_renderer_base(spec, DEFAULT_DISPLAY)


colab_renderer = HTMLRenderer(
    mode="vega",
    fullhtml=True,
    requirejs=False,
    output_div="altair-viz",
    vega_version=VEGA_VERSION,
    vegaembed_version=VEGAEMBED_VERSION,
)


kaggle_renderer = HTMLRenderer(
    mode="vega",
    fullhtml=False,
    requirejs=True,
    vega_version=VEGA_VERSION,
    vegaembed_version=VEGAEMBED_VERSION,
)


html_renderer = HTMLRenderer(
    mode="vega",
    template="universal",
    vega_version=VEGA_VERSION,
    vegaembed_version=VEGAEMBED_VERSION,
)


renderers.register("default", default_renderer)
renderers.register("html", html_renderer)
renderers.register("jupyterlab", default_renderer)
renderers.register("nteract", default_renderer)
renderers.register("colab", colab_renderer)
renderers.register("kaggle", kaggle_renderer)
renderers.register("json", json_renderer)
renderers.enable("default")


class Vega(Displayable):
    """An IPython/Jupyter display class for rendering Vega 5."""

    renderers = renderers
    schema_path = (__name__, "schema/vega-schema.json")


@deprecated(
    "Rendering Vega specifications is deprecated and will be removed in Altair 5."
)
def vega(spec, validate=True):
    """Render and optionally validate a Vega 5 spec.

    This will use the currently enabled renderer to render the spec.

    Parameters
    ==========
    spec: dict
        A fully compliant Vega 5 spec, with the data portion fully processed.
    validate: bool
        Should the spec be validated against the Vega 5 schema?
    """
    from IPython.display import display

    display(Vega(spec, validate=validate))



import pytest

import altair.vegalite.v3 as alt
from altair.vegalite.v3.theme import VEGA_THEMES


@pytest.fixture
def chart():
    return alt.Chart("data.csv").mark_bar().encode(x="x:Q")


def test_vega_themes(chart):
    for theme in VEGA_THEMES:
        with alt.themes.enable(theme):
            dct = chart.to_dict()
        assert dct["usermeta"] == {"embedOptions": {"theme": theme}}
        assert dct["config"] == {
            "view": {"width": 400, "height": 300},
            "mark": {"tooltip": None},
        }



"""
Hexbin Chart
-----------------
This example shows a hexbin chart.
"""
import altair as alt
from vega_datasets import data

source = data.seattle_weather()

# Size of the hexbins
size = 15
# Count of distinct x features
xFeaturesCount = 12
# Count of distinct y features
yFeaturesCount = 7
# Name of the x field
xField = 'date'
# Name of the y field
yField = 'date'

# the shape of a hexagon
hexagon = "M0,-2.3094010768L2,-1.1547005384 2,1.1547005384 0,2.3094010768 -2,1.1547005384 -2,-1.1547005384Z"

alt.Chart(source).mark_point(size=size**2, shape=hexagon).encode(
    x=alt.X('xFeaturePos:Q', axis=alt.Axis(title='Month',
                                           grid=False, tickOpacity=0, domainOpacity=0)),
    y=alt.Y('day(' + yField + '):O', axis=alt.Axis(title='Weekday',
                                                   labelPadding=20, tickOpacity=0, domainOpacity=0)),
    stroke=alt.value('black'),
    strokeWidth=alt.value(0.2),
    fill=alt.Color('mean(temp_max):Q', scale=alt.Scale(scheme='darkblue')),
    tooltip=['month(' + xField + '):O', 'day(' + yField + '):O', 'mean(temp_max):Q']
).transform_calculate(
    # This field is required for the hexagonal X-Offset
    xFeaturePos='(day(datum.' + yField + ') % 2) / 2 + month(datum.' + xField + ')'
).properties(
    # Exact scaling factors to make the hexbins fit
    width=size * xFeaturesCount * 2,
    height=size * yFeaturesCount * 1.7320508076,  # 1.7320508076 is approx. sin(60°)*2
).configure_view(
    strokeWidth=0
)
